import hypermedia.video.*;
import librarytests.*;
import org.openkinect.*;
import org.openkinect.processing.*;
import java.awt.*;

//-------------------------- START HERE --------------------------

Kinect kinect;
OpenCV cv;
PImage img;
ArrayList<Point> hands = new ArrayList<Point>();
ArrayList<Point> data  = new ArrayList<Point>();
int minBlobSize = 2000;
int maxBlobSize = 10000;

//-------------------------- DRAWING VARS --------------------------
int bColor = 0; //background color
int screenWL = 800; //screen width and length
int numberR = 2; //number of ripples
int diaIncreaseRate = 2; //diameter increasing rate
int strokeDecreaseRate = 5; //stroke weight decreasing rate
int alphaS = 60;//alpha value of the stroke
Ripple[] ri = new Ripple[numberR];

/************************************************************
Setup routine - initialize the kinect sensor
************************************************************/
void setup(){
  
  size(640,480);                          //set the size to match sensor resolution
  
  //------------------------- STEP 1: KINECT INIT -----------------------------
  kinect = new Kinect(this);              //create a new sensor object
  kinect.start();                         //start the kinect sensor 
  kinect.enableDepth(true);               //for now enable depth feed
  kinect.tilt(15);                        //adjust the angle of the sensor (yay!)
  
  //------------------------- STEP 2: CV INIT -----------------------------
  cv = new OpenCV(this);
  cv.allocate(width,height);
  
  //------------------------- STEP 3: DRAWING SETUP -----------------------
  for(int i=0; i<numberR; i++)
    ri[i] = new Ripple(0,0,int(random(0,800)),0,int(random(0,255)),int(random(0,255)),int(random(0,255)),alphaS);
}

/************************************************************
Draw routine - actually draw the pixels on the screen
************************************************************/
void draw(){
  
  img = new PImage(width,height);
  
  //------------------------- STEP 1: Process the depth map -----------------------------
  int[] depthMap = kinect.getRawDepth();                      //grab the raw depth map from sensor
  int w = 640, h = 480;                                       //this is the resolution of depth map
  data.clear();
 
  for(int x = 0; x < w; x++){                                 //scan through the entire depth map 
    for(int y = 0; y < h; y++){                               
      int index = y * w + x;
      float depth = rawDepthToMeters(depthMap[index])*1000;  //what's the real depth at point (x,y)?
      if(depth > 300 && depth < 1000) {                       //if depth is within a certain range
        img.set(x,y,255);                                    //mark this point (x,y) on the image
        data.add(new Point(x,y));
      }
    }
  }
  
  //------------------------- STEP 2: Blob extraction -----------------------------
  cv.copy(img);
  Blob[] blobs = cv.blobs(minBlobSize,maxBlobSize*2,5,false);
  println(blobs.length);
  hands.clear();
  for(int i = 0; i < blobs.length; i++){
     Point centroid = blobs[i].centroid;
     hands.add(centroid);     
  }
  
  //------------------------- STEP 3: Draw your stuff -----------------------------
  drawYourStuff();
}


/*********************************************************************
  This is where you can write your drawing code around hand tracking 
 
  Note: the following array lists will be useful
      - data : this is a list of points that show the contour of the hand
      - hands: this is a list of points that locate the hands or 
               other blobs as single points
**********************************************************************/
void drawYourStuff()
{
  //clean screen
  background(bColor,bColor,bColor);
  
  //------------------- YEN CHIA's CODE ----------------  
  if(hands.size() > 0){
      Point pt = hands.get(0);
      
      //draw ripples
      for(int i=0; i<numberR; i++)
      {
        //update kinect
        ri[i].x = pt.x;
        ri[i].y = pt.y;
        //reset
        if(ri[i].sw==0)
        {
          ri[i].x = int(random(0,800));
          ri[i].y = int(random(0,800));
          ri[i].sr = int(random(0,255));
          ri[i].sg = int(random(0,255));
          ri[i].sb = int(random(0,255));
          ri[i].sa = alphaS;
          ri[i].d = 0;
          ri[i].sw = int(random(1,100));
        }
        //update
        ri[i].d += diaIncreaseRate; //increase the diameter
        
        if(ri[i].d%strokeDecreaseRate==0) ri[i].sw--; //decrease the stroke weight
        //render
        strokeWeight(ri[i].sw);
        //fill(255,0);
        stroke(ri[i].sr, ri[i].sg, ri[i].sb, ri[i].sa);
        ellipse(ri[i].x, ri[i].y, ri[i].d, ri[i].d);   
        
        if(ri[i].d > 50){ri[i].sw=0;}
      }  
  }
}

class Ripple
{
  int x,y,d,sw,sr,sg,sb,sa;
  //x: center X coordinate
  //y: center Y coordinate
  //d: diameter
  //sw: stroke weight
  //sr: stroke color R
  //sg: stroke color G
  //sb: stroke color B
  //sa: stroke color alpha
  Ripple (int x_in,int y_in,int d_in,int sw_in,int sr_in,int sg_in,int sb_in,int sa_in)
  { x=x_in; y=y_in; d=d_in; sw=sw_in; sr=sr_in; sg=sg_in; sb=sb_in; sa=sa_in;}
}

//------------------------ YEN CHIA's code ends ---------------------------

/*********************************************************************
  Convert raw depth value to real world depth in meters
**********************************************************************/
float rawDepthToMeters(int depthValue) {
    if (depthValue < 2047) {
      return (float)(1.0 / ((double)(depthValue) * -0.0030711016 + 3.3309495161));
    }
    return 0.0f;
}
