import librarytests.*;
import org.openkinect.*;
import org.openkinect.processing.*;


//-------------------------- START HERE --------------------------

//BasicDepth, KinektHandRipple and kinektOpenCV were all created by Asim Mittal
//for more information, see: http://skillswappingspring12.wordpress.com/2012/04/17/hand-tracking-with-kinect-processing-opencv/
// these examples use the openKinekt library here: 
// and openCV here: 
//note, they only work on Macs, not PCS.
Kinect kinect;

/************************************************************
Setup routine - initialize the kinect sensor
************************************************************/
void setup(){
  size(640,480);                          //set the size to match sensor resolution
  kinect = new Kinect(this);              //create a new sensor object
  kinect.start();                         //start the kinect sensor 
  kinect.enableDepth(true);               //for now enable depth feed
  kinect.tilt(15);                        //adjust the angle of the sensor (yay!)
}

/************************************************************
Draw routine - actually draw the pixels on the screen
************************************************************/
void draw(){
  background(0);                                              //make a black background
  stroke(255);                                                //stroke is white
  
  int[] depthMap = kinect.getRawDepth();                      //grab the raw depth map from sensor
  int w = 640, h = 480;                                       //this is the resolution of depth map
 
  for(int x = 0; x < w; x++){                                 //scan through the entire depth map 
    for(int y = 0; y < h; y++){                               
      int index = y * w + x;
      float depth = rawDepthToMeters(depthMap[index])*1000;  //what's the real depth at point (x,y)?
      if(depth > 500 && depth < 800)point(x,y);              //if the depth at this point is within these constraints, draw it
    }
  }
  
}

/*********************************************************************
  Convert raw depth value to real world depth in meters
**********************************************************************/
float rawDepthToMeters(int depthValue) {
    if (depthValue < 2047) {
      return (float)(1.0 / ((double)(depthValue) * -0.0030711016 + 3.3309495161));
    }
    return 0.0f;
}
