import hypermedia.video.*;
import librarytests.*;
import org.openkinect.*;
import org.openkinect.processing.*;
import java.awt.*;

//-------------------------- START HERE --------------------------

Kinect kinect;
OpenCV cv;
PImage img;
ArrayList<Point> hands = new ArrayList<Point>();
ArrayList<Point> data  = new ArrayList<Point>();
int minBlobSize = 2000;
int maxBlobSize = 10000;

/************************************************************
Setup routine - initialize the kinect sensor
************************************************************/
void setup(){
  
  size(640,480);                          //set the size to match sensor resolution
  
  //------------------------- STEP 1: KINECT INIT -----------------------------
  kinect = new Kinect(this);              //create a new sensor object
  kinect.start();                         //start the kinect sensor 
  kinect.enableDepth(true);               //for now enable depth feed
  kinect.tilt(15);                        //adjust the angle of the sensor (yay!)
  
  //------------------------- STEP 2: CV INIT -----------------------------
  cv = new OpenCV(this);
  cv.allocate(width,height);
}

/************************************************************
Draw routine - actually draw the pixels on the screen
************************************************************/
void draw(){
  
  img = new PImage(width,height);
  
  //------------------------- STEP 1: Process the depth map -----------------------------
  int[] depthMap = kinect.getRawDepth();                      //grab the raw depth map from sensor
  int w = 640, h = 480;                                       //this is the resolution of depth map
  data.clear();
 
  for(int x = 0; x < w; x++){                                 //scan through the entire depth map 
    for(int y = 0; y < h; y++){                               
      int index = y * w + x;
      float depth = rawDepthToMeters(depthMap[index])*1000;  //what's the real depth at point (x,y)?
      if(depth > 300 && depth < 1000) {                       //if depth is within a certain range
        img.set(x,y,255);                                    //mark this point (x,y) on the image
        data.add(new Point(x,y));
      }
    }
  }
  
  //------------------------- STEP 2: Blob extraction -----------------------------
  cv.copy(img);
  Blob[] blobs = cv.blobs(minBlobSize,maxBlobSize*2,5,false);
  println(blobs.length);
  hands.clear();
  for(int i = 0; i < blobs.length; i++){
     Point centroid = blobs[i].centroid;
     hands.add(centroid);     
  }
  
  //------------------------- STEP 3: Draw your stuff -----------------------------
  drawYourStuff();
}

/*********************************************************************
  This is where you can write your drawing code around hand tracking 
 
  Note: the following array lists will be useful
      - data : this is a list of points that show the contour of the hand
      - hands: this is a list of points that locate the hands or 
               other blobs as single points
**********************************************************************/
void drawYourStuff(){
  
  background(0);
  
  //use "data" to draw the outline of the hands
  stroke(80);
  for(int i = 0; i < data.size();i++)  {
    Point pt = data.get(i);
    point(pt.x,pt.y);
  }  
  
  //use "hands" to mark hands with circles
  fill(200);
  for(int i=0; i < hands.size(); i++){
    Point pt = hands.get(i);
    ellipse(pt.x,pt.y,20,20);    
  }
}

/*********************************************************************
  Convert raw depth value to real world depth in meters
**********************************************************************/
float rawDepthToMeters(int depthValue) {
    if (depthValue < 2047) {
      return (float)(1.0 / ((double)(depthValue) * -0.0030711016 + 3.3309495161));
    }
    return 0.0f;
}
